"""
Pyntegrity is Python package that helps checking a file integrity.
Copyright (C) 2022  Salah OSFOR

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
import os
import re
import enum
import hashlib
from pathlib import Path

from .exceptions import FileNotFoundException
from .exceptions import ObjectNotAFileException
from .exceptions import HashStrNotValidException
from .exceptions import MissingFilePermissionException
from .exceptions import HashAlgorithmNotSupportedException

from .config import SUPPORTED_HASH_ALGOS


def detect_hash_algo(checksum_str: str):
    """
    Detects hash algorithm based on its length.

    :param checksum_str: the hash string
    :return: the name of the hash algorithm
    :raises HashAlgorithmNotSupportedException: raised if the hash
        length isn't valid for any supported algorithm
    """
    hash_len = len(checksum_str)
    for name, infos in SUPPORTED_HASH_ALGOS.items():
        if infos["LENGTH"] == hash_len:
            if validate_checksum_str(checksum_str=checksum_str, hash_algo=name):
                return name
    else:
        raise HashAlgorithmNotSupportedException(
            detected_length=hash_len, checksum_str=checksum_str
        )


def validate_checksum_str(checksum_str: str, hash_algo: str):
    """
    Checks if the str is a valid checksum in the detected algorithm

    :param checksum_str:the hash string
    :param hash_algo: the hash algorithm
    :return: True if valid
    :raises HashStrNotValidException: raised if the hash str isn't valid
    """
    hash_name = hash_algo
    pattern = re.compile(SUPPORTED_HASH_ALGOS[hash_name]["REX"])
    if pattern.match(checksum_str):
        return True
    else:
        raise HashStrNotValidException(
            detected_hash_algo=hash_name, checksum_str=checksum_str
        )


def get_file_path_from_str(str_path: str):
    """
    Checks if path is a file and there is read access to it then return a Path object

    :param str_path: provided file path
    :return: file path
    :rtype: Path
    """
    path_object = Path(str_path)
    if path_object.is_file():
        path_object = path_object.resolve()
        if os.access(path_object, os.R_OK):
            return path_object
        else:
            raise MissingFilePermissionException(file_path=str_path)
    elif not path_object.exists():
        raise FileNotFoundException(file_path=str_path)
    else:
        raise ObjectNotAFileException(file_path=str_path)


class IntegrityValidator:
    def __init__(self, str_path: str, checksum_str: str):
        """
        A class to validate file integrity using a checksum string

        :param str_path: the path of the target file
        :param checksum_str:
        """
        self.hash_algo = detect_hash_algo(checksum_str=checksum_str)
        self.checksum_str = checksum_str
        self.file = get_file_path_from_str(str_path)

    def validate_file_integrity(self):
        """
        Function to validate the file integrity
        :return: True is the file valid, False if not
        """
        file_checksum = self.get_file_checksum(self.hash_algo)
        return file_checksum == self.checksum_str

    def get_file_checksum(self, hash_algo: str):
        """
        Calculate file checksum

        :param hash_algo: the name of hash algo that will be used for the checksum
        :return: return file checksum
        """
        hashlib_obj = hashlib.new(hash_algo)
        with self.file.open() as file_to_validate:
            file_content = file_to_validate.read()
            hashlib_obj.update(file_content.encode())
            return hashlib_obj.hexdigest()



"""
Pyntegrity is Python package that helps checking a file integrity.
Copyright (C) 2022  Salah OSFOR

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
# Hash settings

SHA512_LENGTH = 128
SHA256_LENGTH = 64
MD5_LENGTH = 32

SHA512_REX = r"^[A-Fa-f\d]{128}$"
SHA256_REX = r"^[A-Fa-f\d]{64}$"
MD5_REX = r"^[a-fA-F\d]{32}$"

SUPPORTED_HASH_ALGOS = {
    "sha256": {"REX": SHA256_REX, "LENGTH": SHA256_LENGTH},
    "md5": {"REX": MD5_REX, "LENGTH": MD5_LENGTH},
    "sha512": {"REX": SHA512_REX, "LENGTH": SHA512_LENGTH},
}

VERSION = "1.2.0"



from pyntegrity import get_version
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pyntegrity",
    version=get_version(),
    description="Pyntegrity is a Python package that helps you check a file integrity.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ddalu5/pyntegrity",
    author="Salah OSFOR",
    author_email="osfor.salah@gmail.com",
    license="GPL-3.0",
    packages=find_packages(include=["pyntegrity"]),
    test_suite="tests",
    tests_require=["unittest", "black", "pytest", "coverage"],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Information Technology",
        "Topic :: Security",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    zip_safe=False,
)



"""
Pyntegrity is Python package that helps checking a file integrity.
Copyright (C) 2022  Salah OSFOR

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
from .config import SUPPORTED_HASH_ALGOS


class HashAlgorithmNotSupportedException(Exception):
    def __init__(self, detected_length: int, checksum_str: str):
        """
        Exception raised when the hash length does not match
        any supported Hash algorithm;

        :param detected_length: the length of the checked hash string
        :param checksum_str: the hash string
        """
        self.detected_length = detected_length
        self.checksum_str = checksum_str
        self.message = (
            "[!] The hash string length does not "
            "match any supported hash algorithm: ["
        )
        for hash_name, infos in SUPPORTED_HASH_ALGOS.items():
            length = infos["LENGTH"]
            self.message += f" {hash_name}: {length} chars, "
        self.message += "]"

        super().__init__(self.message)


class HashStrNotValidException(Exception):
    def __init__(self, detected_hash_algo: str, checksum_str: str):
        """
        Exception raised when hash string isn't valid
        for the detected hash algorithm;

        :param detected_hash_algo: the name of the detected algo
        :param checksum_str: the hash string
        """
        self.detected_hash_algo = detected_hash_algo
        self.checksum_str = checksum_str
        self.message = (
            f"[!] The hash string doesn't "
            f'seem valid for the detected algorithm"{detected_hash_algo}"'
        )
        super().__init__(self.message)


class FileNotFoundException(Exception):
    def __init__(self, file_path: str):
        """
        Exception raised when the file isn't found using the provided path;

        :param file_path: provided file path
        """
        self.file_path = file_path
        self.message = f'[!] file "{file_path}" not found!'
        super().__init__(self.message)


class ObjectNotAFileException(Exception):
    def __init__(self, file_path: str):
        """
        Exception raised when the provided path isn't a file;

        :param file_path: provided file path
        """
        self.file_path = file_path
        self.message = f'[!] the "{file_path}" is not a valid file!'
        super().__init__(self.message)


class MissingFilePermissionException(Exception):
    def __init__(self, file_path: str):
        """
        Exception raised when the process can't read the file;

        :param file_path: provided file path
        """
        self.file_path = file_path
        self.message = f'[!] the program can not read the file "{file_path}" !'
        super().__init__(self.message)
